@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2021	PgPool Global Development Group
+ * Copyright (c) 2003-2024	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -57,6 +57,19 @@ typedef struct
 	regex_t		regexv;
 }			RegPattern;
 
+typedef enum ProcessManagementModes
+{
+	PM_STATIC = 1,
+	PM_DYNAMIC
+}			ProcessManagementModes;
+
+typedef enum ProcessManagementSstrategies
+{
+	PM_STRATEGY_AGGRESSIVE = 1,
+	PM_STRATEGY_GENTLE,
+	PM_STRATEGY_LAZY
+}			ProcessManagementSstrategies;
+
 typedef enum NativeReplicationSubModes
 {
 	SLONY_MODE = 1,
@@ -119,6 +132,14 @@ typedef enum CHECK_TEMP_TABLE_OPTION
 	CHECK_TEMP_OFF,
 }			CHECK_TEMP_TABLE_OPTION;
 
+/* log_backend_messages */
+typedef enum BGMSG_OPTION
+{
+	BGMSG_NONE = 1,
+	BGMSG_TERSE,
+	BGMSG_VERBOSE,
+}			BGMSG_OPTION;
+
 /*
  * Flags for backendN_flag
  */
@@ -203,14 +224,21 @@ typedef struct
 typedef struct
 {
 	ClusteringModes	backend_clustering_mode;	/* Backend clustering mode */
-	char	   *listen_addresses;	/* hostnames/IP addresses to listen on */
+	ProcessManagementModes	process_management;
+	ProcessManagementSstrategies process_management_strategy;
+	char	   **listen_addresses;	/* hostnames/IP addresses to listen on */
 	int			port;			/* port # to bind */
-	char	   *pcp_listen_addresses;	/* PCP listen address to listen on */
+	char	   **pcp_listen_addresses;	/* PCP listen address to listen on */
 	int			pcp_port;		/* PCP port # to bind */
-	char	   *socket_dir;		/* pgpool socket directory */
+	char	   **unix_socket_directories;		/* pgpool socket directories */
+	char		*unix_socket_group;			/* owner group of pgpool sockets */
+	int			unix_socket_permissions;	/* pgpool sockets permissions */
 	char	   *wd_ipc_socket_dir;	/* watchdog command IPC socket directory */
-	char	   *pcp_socket_dir; /* PCP socket directory */
-	int			num_init_children;	/* # of children initially pre-forked */
+	char	   **pcp_socket_dir; /* PCP socket directory */
+	int			num_init_children;	/* Maximum number of child to
+										 * accept connections */
+	int			min_spare_children;		/* Minimum number of idle children */
+	int			max_spare_children;		/* Minimum number of idle children */
 	int			listen_backlog_multiplier;	/* determines the size of the
 											 * connection queue */
 	int			reserved_connections;	/* # of reserved connections */
@@ -244,6 +272,7 @@ typedef struct
 	bool		replication_mode;	/* replication mode */
 	bool		log_connections;	/* logs incoming connections */
 	bool		log_disconnections;	/* logs closing connections */
+	bool		log_pcp_processes;	/* logs pcp processes */
 	bool		log_hostname;	/* resolve hostname */
 	bool		enable_pool_hba;	/* enables pool_hba.conf file
 									 * authentication */
@@ -305,6 +334,16 @@ typedef struct
 									 * greater than 0 to enable the
 									 * functionality. */
 
+	int		delay_threshold_by_time;	/* If the standby server delays more than
+										* delay_threshold_in_time, any query goes to the
+										* primary only. The unit is in seconds. 0
+										* disables the check. Default is 0.
+										* If delay_threshold_in_time is greater than 0,
+										* delay_threshold will be ignored.
+										* Note that health_check_period required to be
+										* greater than 0 to enable the
+										* functionality. */
+
 	bool		prefer_lower_delay_standby;
 
 	LogStandbyDelayModes log_standby_delay; /* how to log standby lag */
@@ -368,7 +407,10 @@ typedef struct
 	bool		log_statement;	/* logs all SQL statements */
 	bool		log_per_node_statement; /* logs per node detailed SQL
 										 * statements */
+	bool		notice_per_node_statement; /* logs notice message for per node detailed SQL
+										 * statements */
 	bool		log_client_messages;	/* If true, logs any client messages */
+	int			log_backend_messages;	/* logs any backend messages */
 	char	   *lobj_lock_table;	/* table name to lock for rewriting
 									 * lo_creat */
 
@@ -381,6 +423,10 @@ typedef struct
 
 	/* followings till syslog, does not exist in the configuration file */
 	int			num_reset_queries;	/* number of queries in reset_query_list */
+	int			num_listen_addresses;	/* number of entries in listen_addresses */
+	int			num_pcp_listen_addresses;	/* number of entries in pcp_listen_addresses */
+	int			num_unix_socket_directories;	/* number of entries in unix_socket_directories */
+	int			num_pcp_socket_directories;	/* number of entries in pcp_socket_dir */
 	int			num_read_only_function_list;	/* number of functions in
 											 * read_only_function_list */
 	int			num_write_function_list;	/* number of functions in
@@ -466,6 +512,17 @@ typedef struct
 	int			current_memqcache_table_pattern_size;	/* size of the regex
 														 * pattern array */
 
+	/*
+	 * user_redirect_preference_list =
+	 * 'postgres:primary,user[0-4]:1,user[5-9]:2'
+	 */
+	char	   *user_redirect_preference_list;	/* raw string in
+													 * pgpool.conf */
+	RegArray   *redirect_usernames;	/* Precompiled regex patterns for db
+									 * preference list */
+	Left_right_tokens *user_redirect_tokens;	/* db redirect for dbname and node
+											 * string */
+
 	/*
 	 * database_redirect_preference_list =
 	 * 'postgres:primary,mydb[0-4]:1,mydb[5-9]:2'
@@ -527,7 +584,7 @@ typedef struct
 	bool		failover_when_quorum_exists;	/* Do failover only when wd
 												 * cluster holds the quorum */
 	bool		failover_require_consensus; /* Only do failover when majority
-											 * aggrees */
+											 * agrees */
 	bool		allow_multiple_failover_requests_from_node; /* One Pgpool-II node
 															 * can send multiple
 															 * failover requests to
@@ -564,7 +621,8 @@ typedef struct
 	int			pgpool_node_id;	/* pgpool (watchdog) node id */
 	WdNodesConfig wd_nodes;		/* watchdog lists */
 	char	   *trusted_servers;	/* icmp reachable server list (A,B,C) */
-	char	   *delegate_IP;	/* delegate IP address */
+	char	   *trusted_server_command;	/* Executes this command when upper servers are observed */
+	char	   *delegate_ip;	/* delegate IP address */
 	int			wd_interval;	/* lifecheck interval (sec) */
 	char	   *wd_authkey;		/* Authentication key for watchdog
 								 * communication */
@@ -595,7 +653,7 @@ typedef struct
 }			POOL_CONFIG;
 
 extern POOL_CONFIG * pool_config;
-extern char *config_file_dir; /* directory path of config file pgpool.conf */
+extern char config_file_dir[]; /* directory path of config file pgpool.conf */
 
 typedef enum
 {
@@ -614,7 +672,6 @@ typedef struct ConfigVariable
 	struct ConfigVariable *next;
 } ConfigVariable;
 
-
 extern int	pool_init_config(void);
 extern bool pool_get_config(const char *config_file, ConfigContext context);
 extern int	eval_logical(const char *str);
@@ -623,8 +680,5 @@ extern char *backend_status_to_str(BackendInfo * bi);
 
 /* methods used for regexp support */
 extern int	add_regex_pattern(const char *type, char *s);
-extern int	growFunctionPatternArray(RegPattern item);
-extern int	growMemqcacheTablePatternArray(RegPattern item);
-extern int	growQueryPatternArray(RegPattern item);
 
 #endif							/* POOL_CONFIG_H */
