@@ -4,8 +4,8 @@
  *	  creator functions for various nodes. The functions here are for the
  *	  most frequently created nodes.
  *
- * Portions Copyright (c) 2003-2021, PgPool Global Development Group
- * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
+ * Portions Copyright (c) 2003-2024, PgPool Global Development Group
+ * Portions Copyright (c) 1996-2024, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -18,6 +18,8 @@
 
 #include "makefuncs.h"
 #include "utils/palloc.h"
+#include "utils/pgstrcasecmp.h"
+#include "utils/elog.h"
 #include "pg_class.h"
 #include <stdlib.h>
 #include <string.h>
@@ -64,7 +66,7 @@ makeSimpleA_Expr(A_Expr_Kind kind, char *name,
  *	  creates a Var node
  */
 Var *
-makeVar(Index varno,
+makeVar(int varno,
 		AttrNumber varattno,
 		Oid vartype,
 		int32 vartypmod,
@@ -81,12 +83,14 @@ makeVar(Index varno,
 	var->varlevelsup = varlevelsup;
 
 	/*
-	 * Only a few callers need to make Var nodes with varnosyn/varattnosyn
-	 * different from varno/varattno.  We don't provide separate arguments for
-	 * them, but just initialize them to the given varno/varattno.  This
-	 * reduces code clutter and chance of error for most callers.
+	 * Only a few callers need to make Var nodes with non-null varnullingrels,
+	 * or with varnosyn/varattnosyn different from varno/varattno.  We don't
+	 * provide separate arguments for them, but just initialize them to NULL
+	 * and the given varno/varattno.  This reduces code clutter and chance of
+	 * error for most callers.
 	 */
-	var->varnosyn = varno;
+	var->varnullingrels = NULL;
+	var->varnosyn = (Index) varno;
 	var->varattnosyn = varattno;
 
 	/* Likewise, we just set location to "unknown" here */
@@ -102,7 +106,7 @@ makeVar(Index varno,
  *		TargetEntry
  */
 Var *
-makeVarFromTargetEntry(Index varno,
+makeVarFromTargetEntry(int varno,
 					   TargetEntry *tle)
 {
 	return makeVar(varno,
@@ -133,7 +137,7 @@ makeVarFromTargetEntry(Index varno,
  */
 Var *
 makeWholeRowVar(RangeTblEntry *rte,
-				Index varno,
+				int varno,
 				Index varlevelsup,
 				bool allowScalar)
 {
@@ -534,6 +538,22 @@ makeFuncExpr(Oid funcid, Oid rettype, List *args,
 	return funcexpr;
 }
 
+/*
+ * makeStringConst -
+ * 	build a A_Const node of type T_String for given string
+ */
+Node *
+makeStringConst(char *str, int location)
+{
+	A_Const    *n = makeNode(A_Const);
+
+	n->val.sval.type = T_String;
+	n->val.sval.sval = str;
+	n->location = location;
+
+	return (Node *) n;
+}
+
 /*
  * makeDefElem -
  *	build a DefElem node
@@ -628,3 +648,123 @@ makeVacuumRelation(RangeVar *relation, Oid oid, List *va_cols)
 	v->va_cols = va_cols;
 	return v;
 }
+
+/*
+ *  * makeJsonFormat -
+ *   *    creates a JsonFormat node
+ *    */
+JsonFormat *
+makeJsonFormat(JsonFormatType type, JsonEncoding encoding, int location)
+{
+    JsonFormat *jf = makeNode(JsonFormat);
+
+    jf->format_type = type;
+    jf->encoding = encoding;
+    jf->location = location;
+
+    return jf;
+}
+
+/*
+ *  * makeJsonValueExpr -
+ *   *    creates a JsonValueExpr node
+ *    */
+JsonValueExpr *
+makeJsonValueExpr(Expr *raw_expr, Expr *formatted_expr,
+				  JsonFormat *format)
+{
+    JsonValueExpr *jve = makeNode(JsonValueExpr);
+
+    jve->raw_expr = raw_expr;
+    jve->formatted_expr = formatted_expr;
+    jve->format = format;
+
+    return jve;
+}
+
+/*
+ * makeJsonBehavior -
+ *   creates a JsonBehavior node
+ */
+JsonBehavior *
+makeJsonBehavior(JsonBehaviorType btype, Node *expr, int location)
+{
+	JsonBehavior *behavior = makeNode(JsonBehavior);
+
+	behavior->btype = btype;
+	behavior->expr = expr;
+	behavior->location = location;
+
+	return behavior;
+}
+
+/*
+ *  * makeJsonKeyValue -
+ *   *    creates a JsonKeyValue node
+ *    */
+Node *
+makeJsonKeyValue(Node *key, Node *value)
+{
+    JsonKeyValue *n = makeNode(JsonKeyValue);
+
+    n->key = (Expr *) key;
+    n->value = castNode(JsonValueExpr, value);
+
+    return (Node *) n;
+}
+
+/*
+ *  * makeJsonIsPredicate -
+ *   *    creates a JsonIsPredicate node
+ *    */
+Node *
+makeJsonIsPredicate(Node *expr, JsonFormat *format, JsonValueType item_type,
+                    bool unique_keys, int location)
+{
+    JsonIsPredicate *n = makeNode(JsonIsPredicate);
+
+    n->expr = expr;
+    n->format = format;
+    n->item_type = item_type;
+    n->unique_keys = unique_keys;
+    n->location = location;
+
+    return (Node *) n;
+}
+
+/*
+ * makeJsonTablePathSpec -
+ *     Make JsonTablePathSpec node from given path string and name (if any)
+ */
+JsonTablePathSpec *
+makeJsonTablePathSpec(char *string, char *name, int string_location,
+                      int name_location)
+{
+	JsonTablePathSpec *pathspec = makeNode(JsonTablePathSpec);
+
+	Assert(string != NULL);
+	pathspec->string = makeStringConst(string, string_location);
+	if (name != NULL)
+		pathspec->name = pstrdup(name);
+
+	pathspec->name_location = name_location;
+	pathspec->location = string_location;
+
+	return pathspec;
+}
+
+/*
+ * makeJsonTablePath -
+ *     Make JsonTablePath node for given path string and name
+ */
+JsonTablePath *
+makeJsonTablePath(Const *pathvalue, char *pathname)
+{
+	JsonTablePath *path = makeNode(JsonTablePath);
+
+	Assert(IsA(pathvalue, Const));
+	path->value = pathvalue;
+	path->name = pathname;
+
+	return path;
+}
