@@ -140,6 +140,11 @@ struct timeval startTime;
 bool		stop_now = false;
 #endif
 
+/*
+ * If true, connection pool has been initialized.
+ */
+static bool connection_pool_initialized = false;
+
 /*
 * child main loop
 */
@@ -223,6 +228,7 @@ do_child(int *fds)
 	{
 		child_exit(POOL_EXIT_AND_RESTART);
 	}
+	connection_pool_initialized = true;
 
 	/*
 	 * Open pool_passwd in child process.  This is necessary to avoid the file
@@ -477,7 +483,12 @@ do_child(int *fds)
 
 		/* Mark this connection pool is not connected from frontend */
 		pool_coninfo_unset_frontend_connected(pool_get_process_context()->proc_id, pool_pool_index());
+
+		/*
+		 * Update number of established connections in the connection pool.
+		 */
 		update_pooled_connection_count();
+
 		accepted = 0;
 		connection_count_down();
 		if (pool_config->log_disconnections)
@@ -524,7 +535,7 @@ backend_cleanup(POOL_CONNECTION * volatile *frontend, POOL_CONNECTION_POOL * vol
 	sp = MAIN_CONNECTION(backend)->sp;
 
 	/*
-	 * cach connection if connection cache configuration parameter is enabled
+	 * cache connection if connection cache configuration parameter is enabled
 	 * and frontend connection is not invalid
 	 */
 	if (sp && pool_config->connection_cache != 0 && frontend_invalid == false)
@@ -1069,9 +1080,8 @@ static POOL_CONNECTION_POOL * connect_backend(StartupPacket *sp, POOL_CONNECTION
 
 		for (i = 0; i < NUM_BACKENDS; i++)
 		{
-			if (VALID_BACKEND(i))
+			if (VALID_BACKEND(i) && CONNECTION_SLOT(backend, i))
 			{
-
 				/* set DB node id */
 				pool_set_db_node_id(CONNECTION(backend, i), i);
 
@@ -1194,6 +1204,7 @@ static RETSIGTYPE close_idle_connection(int sig)
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 	ConnectionInfo *info;
 	int			save_errno = errno;
+	int			main_node_id;
 
 	/*
 	 * DROP DATABASE is ongoing.
@@ -1201,44 +1212,35 @@ static RETSIGTYPE close_idle_connection(int sig)
 	if (ignore_sigusr1)
 		return;
 
-#ifdef NOT_USED
-	ereport(DEBUG1,
-			(errmsg("close connection request received")));
-#endif
-
 	for (j = 0; j < pool_config->max_pool; j++, p++)
 	{
-		if (!MAIN_CONNECTION(p))
-			continue;
-		if (!MAIN_CONNECTION(p)->sp)
-			continue;
-		if (MAIN_CONNECTION(p)->sp->user == NULL)
+		main_node_id = in_use_backend_id(p);
+		if (main_node_id < 0)
 			continue;
 
-		if (MAIN_CONNECTION(p)->closetime > 0)	/* idle connection? */
+		if (!CONNECTION_SLOT(p, main_node_id))
+			continue;
+		if (!CONNECTION_SLOT(p, main_node_id)->sp)
+			continue;
+		if (CONNECTION_SLOT(p, main_node_id)->sp->user == NULL)
+			continue;
+
+		if (CONNECTION_SLOT(p, main_node_id)->closetime > 0)	/* idle connection? */
 		{
-#ifdef NOT_USED
-			ereport(DEBUG1,
-					(errmsg("closing idle connection"),
-					 errdetail("user: %s database: %s", MAIN_CONNECTION(p)->sp->user, MAIN_CONNECTION(p)->sp->database)));
-#endif
+			bool	freed = false;
 
 			pool_send_frontend_exits(p);
 
 			for (i = 0; i < NUM_BACKENDS; i++)
 			{
-				if (!VALID_BACKEND(i))
+				if (!CONNECTION_SLOT(p, i))
 					continue;
 
-				if (i == 0)
+				if (!freed)
 				{
-					/*
-					 * only first backend allocated the memory for the start
-					 * up packet
-					 */
 					pool_free_startup_packet(CONNECTION_SLOT(p, i)->sp);
 					CONNECTION_SLOT(p, i)->sp = NULL;
-
+					freed = true;
 				}
 				pool_close(CONNECTION(p, i));
 			}
@@ -1266,7 +1268,7 @@ static RETSIGTYPE authentication_timeout(int sig)
 }
 
 /*
- * Enable authtentication timeout.
+ * Enable authentication timeout.
  */
 static void
 enable_authentication_timeout(void)
@@ -1278,7 +1280,7 @@ enable_authentication_timeout(void)
 }
 
 /*
- * Disable authtentication timeout.
+ * Disable authentication timeout.
  */
 static void
 disable_authentication_timeout(void)
@@ -1355,7 +1357,7 @@ child_will_go_down(int code, Datum arg)
 	}
 
 	/* let backend know now we are exiting */
-	if (pool_connection_pool)
+	if (connection_pool_initialized)
 		close_all_backend_connections();
 }
 
@@ -1803,9 +1805,10 @@ check_config_reload(void)
 		if (pool_config->enable_pool_hba)
 		{
 			load_hba(get_hba_file_name());
-			if (strcmp("", pool_config->pool_passwd))
-				pool_reopen_passwd_file();
 		}
+		if (strcmp("", pool_config->pool_passwd))
+			pool_reopen_passwd_file();
+
 		got_sighup = 0;
 	}
 }
