@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2020	PgPool Global Development Group
+ * Copyright (c) 2003-2024	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -43,6 +43,7 @@
 #include <stdlib.h>
 
 #include "pool.h"
+#include "context/pool_query_context.h"
 #include "utils/pool_stream.h"
 #include "utils/palloc.h"
 #include "pool_config.h"
@@ -67,6 +68,8 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p);
 static int	check_socket_status(int fd);
 static bool connect_with_timeout(int fd, struct addrinfo *walk, char *host, int port, bool retry);
 
+#define TMINTMAX 0x7fffffff
+
 /*
 * initialize connection pools. this should be called once at the startup.
 */
@@ -158,6 +161,13 @@ pool_get_cp(char *user, char *database, int protoMajor, int check_socket)
 					ereport(LOG,
 							(errmsg("connection closed."),
 							 errdetail("retry to create new connection pool")));
+					/*
+					 * It is possible that one of backend just broke.  sleep 1
+					 * second to wait for failover occurres, then wait for the
+					 * failover finishes.
+					 */
+					sleep(1);
+					wait_for_failover_to_finish();
 
 					for (j = 0; j < NUM_BACKENDS; j++)
 					{
@@ -247,6 +257,7 @@ pool_create_cp(void)
 	POOL_CONNECTION_POOL *oldestp;
 	POOL_CONNECTION_POOL *ret;
 	ConnectionInfo *info;
+	int		main_node_id;
 
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 
@@ -259,7 +270,7 @@ pool_create_cp(void)
 
 	for (i = 0; i < pool_config->max_pool; i++)
 	{
-		if (MAIN_CONNECTION(p) == NULL)
+		if (in_use_backend_id(p) < 0)	/* is this connection pool out of use? */
 		{
 			ret = new_connection(p);
 			if (ret)
@@ -277,21 +288,25 @@ pool_create_cp(void)
 	 * discard it.
 	 */
 	oldestp = p = pool_connection_pool;
-	closetime = MAIN_CONNECTION(p)->closetime;
+	closetime = TMINTMAX;
 	pool_index = 0;
 
 	for (i = 0; i < pool_config->max_pool; i++)
 	{
+		main_node_id = in_use_backend_id(p);
+		if (main_node_id < 0)
+			elog(ERROR, "no in use backend found");	/* this should not happen */
+
 		ereport(DEBUG1,
 				(errmsg("creating connection pool"),
 				 errdetail("user: %s database: %s closetime: %ld",
-						   MAIN_CONNECTION(p)->sp->user,
-						   MAIN_CONNECTION(p)->sp->database,
-						   MAIN_CONNECTION(p)->closetime)));
+						   CONNECTION_SLOT(p, main_node_id)->sp->user,
+						   CONNECTION_SLOT(p, main_node_id)->sp->database,
+						   CONNECTION_SLOT(p, main_node_id)->closetime)));
 
-		if (MAIN_CONNECTION(p)->closetime < closetime)
+		if (CONNECTION_SLOT(p, main_node_id)->closetime < closetime)
 		{
-			closetime = MAIN_CONNECTION(p)->closetime;
+			closetime = CONNECTION_SLOT(p, main_node_id)->closetime;
 			oldestp = p;
 			pool_index = i;
 		}
@@ -299,18 +314,21 @@ pool_create_cp(void)
 	}
 
 	p = oldestp;
+	main_node_id = in_use_backend_id(p);
+	if (main_node_id < 0)
+		elog(ERROR, "no in use backend found");	/* this should not happen */
 	pool_send_frontend_exits(p);
 
 	ereport(DEBUG1,
 			(errmsg("creating connection pool"),
 			 errdetail("discarding old %zd th connection. user: %s database: %s",
 					   oldestp - pool_connection_pool,
-					   MAIN_CONNECTION(p)->sp->user,
-					   MAIN_CONNECTION(p)->sp->database)));
+					   CONNECTION_SLOT(p, main_node_id)->sp->user,
+					   CONNECTION_SLOT(p, main_node_id)->sp->database)));
 
 	for (i = 0; i < NUM_BACKENDS; i++)
 	{
-		if (!VALID_BACKEND(i))
+		if (CONNECTION_SLOT(p, i) == NULL)
 			continue;
 
 		if (!freed)
@@ -391,8 +409,6 @@ pool_backend_timer_handler(int sig)
 void
 pool_backend_timer(void)
 {
-#define TMINTMAX 0x7fffffff
-
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 	int			i,
 				j;
@@ -471,7 +487,7 @@ pool_backend_timer(void)
 			nearest = 1;
 		pool_alarm(pool_backend_timer_handler, nearest);
 	}
-
+	update_pooled_connection_count();
 	POOL_SETMASK(&UnBlockSig);
 }
 
@@ -868,6 +884,7 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 	int			active_backend_count = 0;
 	int			i;
 	bool		status_changed = false;
+	volatile BACKEND_STATUS	status;
 
 	MemoryContext oldContext = MemoryContextSwitchTo(TopMemoryContext);
 
@@ -892,8 +909,8 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 		 * that the local status is up, while the global status has been
 		 * changed to down by failover.
 		 */
-		if (BACKEND_INFO(i).backend_status != CON_UP &&
-			BACKEND_INFO(i).backend_status != CON_CONNECT_WAIT)
+		status = BACKEND_INFO(i).backend_status;
+		if (status != CON_UP && status != CON_CONNECT_WAIT)
 		{
 			ereport(DEBUG1,
 					(errmsg("creating new connection to backend"),
@@ -901,7 +918,8 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 							   i, BACKEND_INFO(i).backend_status)));
 
 			/* sync local status with global status */
-			*(my_backend_status[i]) = BACKEND_INFO(i).backend_status;
+			*(my_backend_status[i]) = status;
+			my_main_node_id = Req_info->main_node_id;
 			continue;
 		}
 
@@ -1055,14 +1073,54 @@ close_all_backend_connections(void)
 
 	for (i = 0; i < pool_config->max_pool; i++, p++)
 	{
-		if (!MAIN_CONNECTION(p))
+		int	backend_id = in_use_backend_id(p);
+
+		if (backend_id < 0)
 			continue;
-		if (!MAIN_CONNECTION(p)->sp)
+		if (CONNECTION_SLOT(p, backend_id) == NULL)
 			continue;
-		if (MAIN_CONNECTION(p)->sp->user == NULL)
+		if (CONNECTION_SLOT(p, backend_id)->sp == NULL)
+			continue;
+		if (CONNECTION_SLOT(p, backend_id)->sp->user == NULL)
 			continue;
 		pool_send_frontend_exits(p);
 	}
 
 	POOL_SETMASK(&oldmask);
 }
+
+/*
+ * Return number of established connections in the connection pool.
+ * This is called when a client disconnects to pgpool.
+ */
+void
+update_pooled_connection_count(void)
+{
+	int i;
+	int count = 0;
+	POOL_CONNECTION_POOL *p = pool_connection_pool;
+	for (i = 0; i < pool_config->max_pool; i++, p++)
+	{
+		if (MAIN_CONNECTION(p))
+			count++;
+	}
+	pool_get_my_process_info()->pooled_connections = count;
+}
+
+/*
+ * Return the first node id in use.
+ * If no node is in use, return -1.
+ */
+int
+in_use_backend_id(POOL_CONNECTION_POOL *pool)
+{
+	int	i;
+
+	for (i = 0; i < NUM_BACKENDS; i++)
+	{
+		if (pool->slots[i])
+			return i;
+	}
+
+	return -1;
+}
