@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2021	PgPool Global Development Group
+ * Copyright (c) 2003-2024	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -58,17 +58,19 @@ typedef enum
 {
 	PCP_ATTACH_NODE,
 	PCP_DETACH_NODE,
+	PCP_HEALTH_CHECK_STATS,
+	PCP_INVALIDATE_QUERY_CACHE,
+	PCP_LOG_ROTATE,
 	PCP_NODE_COUNT,
 	PCP_NODE_INFO,
-	PCP_HEALTH_CHECK_STATS,
 	PCP_POOL_STATUS,
 	PCP_PROC_COUNT,
 	PCP_PROC_INFO,
 	PCP_PROMOTE_NODE,
 	PCP_RECOVERY_NODE,
-	PCP_STOP_PGPOOL,
-	PCP_WATCHDOG_INFO,
 	PCP_RELOAD_CONFIG,
+	PCP_WATCHDOG_INFO,
+	PCP_STOP_PGPOOL,
 	UNKNOWN,
 }			PCP_UTILITIES;
 
@@ -95,6 +97,8 @@ struct AppTypes AllAppTypes[] =
 	{"pcp_stop_pgpool", PCP_STOP_PGPOOL, "m:h:p:U:s:wWvda", "terminate pgpool-II"},
 	{"pcp_watchdog_info", PCP_WATCHDOG_INFO, "n:h:p:U:wWvd", "display a pgpool-II watchdog's information"},
 	{"pcp_reload_config",PCP_RELOAD_CONFIG,"h:p:U:s:wWvd", "reload a pgpool-II config file"},
+	{"pcp_log_rotate",PCP_LOG_ROTATE,"h:p:U:s:wWvd", "rotate the Pgpool-II's log file"},
+	{"pcp_invalidate_query_cache",PCP_INVALIDATE_QUERY_CACHE,"h:p:U:s:wWvd", "invalidate query cache"},
 	{NULL, UNKNOWN, NULL, NULL},
 };
 struct AppTypes *current_app_type;
@@ -115,7 +119,7 @@ main(int argc, char **argv)
 	int			i;
 	bool		all = false;
 	bool		debug = false;
-	bool		need_password = true;
+	bool		need_password = false;
 	bool		gracefully = false;
 	bool		switchover = false;
 	bool		verbose = false;
@@ -393,6 +397,11 @@ main(int argc, char **argv)
 			pcpResInfo = pcp_detach_node(pcpConn, nodeID);
 	}
 
+	else if (current_app_type->app_type == PCP_LOG_ROTATE)
+	{
+		pcpResInfo = pcp_log_rotate(pcpConn,command_scope);
+	}
+
 	else if (current_app_type->app_type == PCP_NODE_COUNT)
 	{
 		pcpResInfo = pcp_node_count(pcpConn);
@@ -408,6 +417,11 @@ main(int argc, char **argv)
 		pcpResInfo = pcp_health_check_stats(pcpConn, nodeID);
 	}
 
+	else if (current_app_type->app_type == PCP_INVALIDATE_QUERY_CACHE)
+	{
+		pcpResInfo = pcp_invalidate_query_cache(pcpConn);
+	}
+
 	else if (current_app_type->app_type == PCP_POOL_STATUS)
 	{
 		pcpResInfo = pcp_pool_status(pcpConn);
@@ -436,6 +450,11 @@ main(int argc, char **argv)
 		pcpResInfo = pcp_recovery_node(pcpConn, nodeID);
 	}
 
+	else if (current_app_type->app_type == PCP_RELOAD_CONFIG)
+	{
+		pcpResInfo = pcp_reload_config(pcpConn,command_scope);
+	}
+
 	else if (current_app_type->app_type == PCP_STOP_PGPOOL)
 	{
 		pcpResInfo = pcp_terminate_pgpool(pcpConn, shutdown_mode, command_scope);
@@ -446,11 +465,6 @@ main(int argc, char **argv)
 		pcpResInfo = pcp_watchdog_info(pcpConn, nodeID);
 	}
 
-	else if (current_app_type->app_type == PCP_RELOAD_CONFIG)
-	{
-		pcpResInfo = pcp_reload_config(pcpConn,command_scope);
-	}
-
 	else
 	{
 		/* should never happen */
@@ -522,17 +536,18 @@ output_nodeinfo_result(PCPResultInfo * pcpResInfo, bool all, bool verbose)
 	struct tm	tm;
 	char	   *frmt;
 	int         array_size = pcp_result_slot_count(pcpResInfo);
+	char		standby_delay_str[64];
 
 	if (verbose)
 	{
 		const char *titles[] = {"Hostname", "Port", "Status", "Weight", "Status Name", "Backend Status Name", "Role", "Backend Role", "Replication Delay", "Replication State", "Replication Sync State", "Last Status Change"};
-		const char *types[] = {"s", "d", "d", "f", "s", "s", "s", "s", "lu", "s", "s", "s"};
+		const char *types[] = {"s", "d", "d", "f", "s", "s", "s", "s", "s", "s", "s", "s"};
 
 		frmt = format_titles(titles, types, sizeof(titles)/sizeof(char *));
 	}
 	else
 	{
-		frmt = "%s %d %d %f %s %s %s %s %lu %s %s %s\n";
+		frmt = "%s %d %d %f %s %s %s %s %s %s %s %s\n";
 	}
 
 	for (i = 0; i < array_size; i++)
@@ -549,6 +564,22 @@ output_nodeinfo_result(PCPResultInfo * pcpResInfo, bool all, bool verbose)
 		localtime_r(&backend_info->status_changed_time, &tm);
 		strftime(last_status_change, sizeof(last_status_change), "%F %T", &tm);
 
+		if (backend_info->standby_delay_by_time)
+		{
+			snprintf(standby_delay_str, sizeof(standby_delay_str), "%.6f", ((float)backend_info->standby_delay)/1000000);
+			if (verbose)
+			{
+				if (backend_info->standby_delay >= 2*1000*1000)
+					strcat(standby_delay_str, " seconds");
+				else
+					strcat(standby_delay_str, " second");
+			}
+		}
+		else
+		{
+			snprintf(standby_delay_str, sizeof(standby_delay_str), "%lu", backend_info->standby_delay);
+		}
+
 		printf(frmt,
 			   backend_info->backend_hostname,
 			   backend_info->backend_port,
@@ -558,7 +589,7 @@ output_nodeinfo_result(PCPResultInfo * pcpResInfo, bool all, bool verbose)
 			   backend_info->pg_backend_status,
 			   role_to_str(backend_info->role),
 			   backend_info->pg_role,
-			   backend_info->standby_delay,
+			   standby_delay_str,
 			   backend_info->replication_state[0] == '\0' ? "none" : backend_info->replication_state,
 			   backend_info->replication_sync_state[0] == '\0' ? "none" : backend_info->replication_sync_state,
 			   last_status_change);
@@ -704,13 +735,14 @@ output_procinfo_result(PCPResultInfo * pcpResInfo, bool all, bool verbose)
 		"Database", "Username", "Start time", "Client connection count",
 		"Major", "Minor", "Backend connection time", "Client connection time",
 		"Client idle duration", "Client disconnection time", "Pool Counter", "Backend PID",
-		"Connected", "PID", "Backend ID", "Status"
+		"Connected", "PID", "Backend ID", "Status", "Load balance node"
 	};
 	const char *types[] = {
 		"s", "s", "s", "s",
 		"s", "s", "s", "s",
 		"s", "s", "s", "s",
-		"s", "s", "s", "s"
+		"s", "s", "s", "s",
+		"s"
 	};
 
 
@@ -718,7 +750,7 @@ output_procinfo_result(PCPResultInfo * pcpResInfo, bool all, bool verbose)
 		format = format_titles(titles, types, sizeof(titles)/sizeof(char *));
 	else
 	{
-		format = "%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n";
+		format = "%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n";
 	}
 
 	for (i = 0; i < array_size; i++)
@@ -728,7 +760,7 @@ output_procinfo_result(PCPResultInfo * pcpResInfo, bool all, bool verbose)
 
 		if (pools == NULL)
 			break;
-		if ((!all) && (pools->database[0] == '\0'))
+		if (((!all) && (pools->database[0] == '\0')) || (pools->pool_pid[0] == '\0'))
 			continue;
 		printed = true;
 		printf(format,
@@ -747,7 +779,8 @@ output_procinfo_result(PCPResultInfo * pcpResInfo, bool all, bool verbose)
 			   pools->pool_connected,
 			   pools->pool_pid,
 			   pools->backend_id,
-			   pools->status);
+			   pools->status,
+			   pools->load_balance_node);
 	}
 	if (printed == false)
 		printf("No process information available\n\n");
@@ -842,7 +875,8 @@ static inline bool
 app_support_cluster_mode(void)
 {
 	return (current_app_type->app_type == PCP_STOP_PGPOOL ||
-			current_app_type->app_type == PCP_RELOAD_CONFIG);
+			current_app_type->app_type == PCP_RELOAD_CONFIG ||
+			current_app_type->app_type == PCP_LOG_ROTATE);
 }
 
 static void
